<?php

namespace Drupal\audio_wavesurfer_clips\Plugin\Field\FieldFormatter;

use Drupal\audio_clips\Entity\AudioClipType;
use Drupal\audio_wavesurfer\Plugin\Field\FieldFormatter\AudioWavesurferFormatter;
use Drupal\Core\Field\Attribute\FieldFormatter;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\StringTranslation\TranslatableMarkup;
use Drupal\Core\Field\FieldItemListInterface;
use Drupal\file\Entity\File;

/**
 * Plugin 'audio_wavesurfer_clip' formatter.
 */
#[FieldFormatter(
  id: 'audio_wavesurfer_clip_formatter',
  label: new TranslatableMarkup('Audio Wavesurfer Clip Formatter'),
  description: new TranslatableMarkup('Display the audio file with Wavesurfer library and specific clip type.'),
  field_types: [
    'file',
  ],
)]
class AudioWavesurferClipFormatter extends AudioWavesurferFormatter {

  /**
   * {@inheritdoc}
   */
  public static function defaultSettings() {
    return [
      'audio_clip_type_display' => '',
    ] + parent::defaultSettings();
  }

  /**
   * {@inheritdoc}
   */
  public function settingsForm(array $form, FormStateInterface $form_state) {
    $form = parent::settingsForm($form, $form_state);
    $audio_clip_names = AudioClipType::getClipTypeNames();

    $form['audio_clip_type_display'] = [
      '#type' => 'select',
      '#title' => t('Audio Clip Type'),
      '#empty_option' => $this->t('None (original audio)'),
      '#default_value' => $this->getSetting('audio_clip_type_display'),
      '#options' => $audio_clip_names,
    ];

    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function settingsSummary() {
    $summary = parent::settingsSummary();
    if ($this->getSetting('audio_clip_type_display')) {
      $summary[] = $this->t('Audio Clip display: %clip', ['%clip' => $this->getSetting('audio_clip_type_display')]);
    }
    else {
      $summary[] = $this->t('Original audio');
    }
    return $summary;
  }

  /**
   * {@inheritdoc}
   */
  public function viewElements(FieldItemListInterface $items, $langcode) {
    $element = parent::viewElements($items, $langcode);

    foreach ($element as $delta => $item) {
      $audio_clip_fid = $item['#file_upload_id'];

      $clip_name = $this->getSetting('audio_clip_type_display');
      if (!empty($clip_name)) {
        $query = \Drupal::entityQuery('audio_clip')
          ->accessCheck(TRUE)
          ->condition('type', $clip_name)
          ->condition('original_id', $audio_clip_fid)
          ->execute();

        $storage = \Drupal::entityTypeManager()->getStorage('audio_clip');
        $audio_clip = $storage->load(reset($query));
        if (!$audio_clip) {
          continue;
        }

        $audio_clip_fid = $audio_clip->target_id->value;
        $audio_file = File::load($audio_clip_fid);
        $audio_file_path = \Drupal::service('file_url_generator')->generateString($audio_file->getFileUri());

        $element[$delta]['#file_upload_url'] = $audio_file_path;
        $element[$delta]['#file_upload_id'] = $audio_clip_fid;

        // Manage waveform JSON.
        $waveform_options = \Drupal::config('audio_wavesurfer.settings')->get('waveform_options');
        if (array_key_exists('waveform_usage', $waveform_options) && $waveform_options['waveform_usage']) {
          $waveform_peaks = \Drupal::service('audio_wavesurfer.waveform_json_service')->getAudioWaveformStored($audio_clip_fid);
          if ($waveform_peaks) {
            $element[$delta]['#attached']['drupalSettings'] = [
              'audio_wavesurfer' => [
                'waveform_peaks' => $waveform_peaks,
              ],
            ];
          }
        }
      }
    }

    return $element;
  }

}
