<?php

namespace Drupal\audio_wavesurfer_clips\Plugin\Field\FieldWidget;

use Drupal\audio_clips\Entity\AudioClipType;
use Drupal\audio_clips\FFMpeg\AudioFile;
use Drupal\Core\Field\Attribute\FieldWidget;
use Drupal\Core\Field\FieldItemListInterface;
use Drupal\Core\StringTranslation\TranslatableMarkup;
use Drupal\Core\Form\FormStateInterface;
use Drupal\file\Entity\File;
use Drupal\file\Plugin\Field\FieldWidget\FileWidget;

/**
 * Plugin implementation of the 'audio_clip_wavesurfer' widget.
 */
#[FieldWidget(
  id: 'audio_wavesurfer_clip_widget',
  label: new TranslatableMarkup('Audio Wavesurfer Clip Widget'),
  description: new TranslatableMarkup('Display the Audio Wavesurfer Clip Widget.'),
  field_types: [
    'file',
  ],
)]
class AudioWavesurferClipWidget extends FileWidget {

  /**
   * {@inheritdoc}
   */
  public static function defaultSettings() {
    return [
      'audio_clip_type' => [],
    ] + parent::defaultSettings();
  }

  /**
   * {@inheritdoc}
   */
  public function settingsForm(array $form, FormStateInterface $form_state) {
    $element = parent::settingsForm($form, $form_state);
    $audio_clip_names = AudioClipType::getClipTypeNames();

    $element['audio_clip_type'] = [
      '#type' => 'select',
      '#title' => $this->t('Audio Clip Type'),
      '#default_value' => $this->getSetting('audio_clip_type'),
      '#options' => $audio_clip_names,
      '#multiple' => TRUE,
    ];

    return $element;
  }

  /**
   * {@inheritdoc}
   */
  public function settingsSummary() {
    $summary = [];

    $audio_clip = $this->getSetting('audio_clip_type');
    if (!empty($audio_clip)) {
      $summary[] = $this->t('Audio clip type: @clip', ['@clip' => implode(", ", $audio_clip)]);
    }
    else {
      $summary[] = $this->t('No audio clip type');
    }

    return $summary;
  }

  /**
   * {@inheritdoc}
   */
  public function formElement(FieldItemListInterface $items, $delta, array $element, array &$form, FormStateInterface $form_state) {
    $element = parent::formElement($items, $delta, $element, $form, $form_state);

    foreach ($items as $item) {
      $file_upload_id = $item->target_id;
      if ($file_upload_id === NULL && $form_state->getValue('field_media_audio_file')) {
        $field_media_audio_file = $form_state->getValue('field_media_audio_file');
        $file_upload_id = $field_media_audio_file[0]['fids'][0];
      }

      if (!$file_upload_id) {
        return $element;
      }

      $file_upload = File::load($file_upload_id);
      $audio_file = new AudioFile(\Drupal::service('file_system')->realpath($file_upload->getFileUri()));
      $file_upload_url = \Drupal::service('file_url_generator')->generateString($file_upload->getFileUri());
      $file_upload_duration = $audio_file->getAudioDuration();

      $audio_clip_types = $this->prepareAudioClipType($this->getSetting('audio_clip_type'));
      if (empty($audio_clip_types)) {
        return $element;
      }

      $fids_in_audio_clip = \Drupal::entityQuery('audio_clip')
        ->accessCheck(TRUE)
        ->condition('original_id', $file_upload_id)
        ->condition('type', array_keys($audio_clip_types), 'IN')
        ->execute();

      $audioClipStorage = \Drupal::entityTypeManager()->getStorage('audio_clip');
      /** @var \Drupal\audio_clips\Entity\AudioClip $audio_clips_storage */
      $audio_clips_storage = $audioClipStorage->loadMultiple($fids_in_audio_clip);

      $audio_clips_by_type = [];
      foreach ($audio_clips_storage as $audio_clip_storage) {
        $type_value = $audio_clip_storage->get('type')->target_id;
        $audio_clips_by_type[$type_value] = $audio_clip_storage;
      }

      $element['audio_wavesurfer_clip'] = [
        '#type' => 'audio_wavesurfer_clip',
        '#audio_clip_type' => $audio_clip_types,
        '#audio_clip_storage' => $audio_clips_by_type,
        '#file_upload_id' => $file_upload_id,
        '#file_upload_url' => $file_upload_url,
        '#file_upload_duration' => $file_upload_duration,
      ];
    }

    return $element;
  }

  /**
   * Return formatted audio_clip_type with label and duration.
   *
   * @param array $audio_clip_types_settings
   *   The audio clip types selected by user.
   *
   * @return array
   *   return prepared audio clip type.
   */
  private function prepareAudioClipType(array $audio_clip_types_settings): array {
    $audio_clip_types = AudioClipType::getAllClipType();
    $audio_clip_types_formatted = [];
    foreach ($audio_clip_types_settings as $audio_clip_type_settings) {
      $audio_clip_types_formatted[$audio_clip_type_settings] = [
        'label' => $audio_clip_types[$audio_clip_type_settings]->label(),
        'min_duration' => $audio_clip_types[$audio_clip_type_settings]->getMinDuration(),
        'max_duration' => $audio_clip_types[$audio_clip_type_settings]->getMaxDuration(),
      ];
    }
    return $audio_clip_types_formatted;
  }

}
