<?php

namespace Drupal\audio_wavesurfer_clips\Services;

use Drupal\audio_clips\Services\AudioClipService;
use Drupal\audio_wavesurfer\Services\WaveformJsonService;
use Drupal\Core\Entity\EntityInterface;
use Drupal\Core\Entity\EntityStorageInterface;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\File\FileSystemInterface;
use Drupal\Core\Session\AccountInterface;
use Drupal\file\FileUsage\FileUsageInterface;

/**
 * Service to manage audio-clip creation.
 */
class AudioPrepareClipService {

  /**
   * The entity type manager service.
   *
   * @var \Drupal\Core\Entity\EntityTypeManagerInterface
   */
  protected EntityTypeManagerInterface $entityTypeManager;

  /**
   * The clip storage.
   *
   * @var \Drupal\Core\Entity\EntityStorageInterface
   */
  protected EntityStorageInterface $audioClipStorage;

  /**
   * The current User.
   *
   * @var \Drupal\Core\Session\AccountInterface
   */
  protected AccountInterface $currentUser;

  /**
   * The file system.
   *
   * @var \Drupal\Core\File\FileSystemInterface
   */
  protected FileSystemInterface $fileSystem;

  /**
   * The file usage service.
   *
   * @var \Drupal\file\FileUsage\FileUsageInterface
   */
  protected FileUsageInterface $fileUsage;

  /**
   * The audio clip service.
   *
   * @var \Drupal\audio_clips\Services\AudioClipService
   */
  protected AudioClipService $audioClipService;

  /**
   * The audio clip service.
   *
   * @var \Drupal\audio_wavesurfer\Services\WaveformJsonService
   */
  protected WaveformJsonService $waveformJsonService;

  /**
   * AudioClipService constructor.
   *
   * @param \Drupal\Core\Entity\EntityTypeManagerInterface $entity_type_manager
   *   The entity type manager interface.
   * @param \Drupal\Core\Session\AccountInterface $current_user
   *   The account interface.
   * @param \Drupal\Core\File\FileSystemInterface $file_system
   *   The file system inteface.
   * @param \Drupal\file\FileUsage\FileUsageInterface $file_usage
   *   The file usagge inteface.
   * @param \Drupal\audio_clips\Services\AudioClipService $audio_clip_service
   *   The audio clip service.
   * @param \Drupal\audio_wavesurfer\Services\WaveformJsonService $waveform_json_service
   *   The waveform json service.
   *
   * @throws \Drupal\Component\Plugin\Exception\InvalidPluginDefinitionException
   * @throws \Drupal\Component\Plugin\Exception\PluginNotFoundException
   */
  public function __construct(
    EntityTypeManagerInterface $entity_type_manager,
    AccountInterface $current_user,
    FileSystemInterface $file_system,
    FileUsageInterface $file_usage,
    AudioClipService $audio_clip_service,
    WaveformJsonService $waveform_json_service,
  ) {
    $this->entityTypeManager = $entity_type_manager;
    $this->audioClipStorage = $this->entityTypeManager->getStorage('audio_clip');
    $this->currentUser = $current_user;
    $this->fileSystem = $file_system;
    $this->fileUsage = $file_usage;
    $this->audioClipService = $audio_clip_service;
    $this->waveformJsonService = $waveform_json_service;
  }

  /**
   * Build Audio clip to entity.
   *
   * @param \Drupal\Core\Entity\EntityInterface $entity
   *   The audio-clip entity.
   *
   * @return void
   *   Return void.
   *
   * @throws \Drupal\Core\Entity\EntityStorageException
   * @throws \Exception
   */
  public function buildAudioClipToEntity(EntityInterface $entity): void {
    // @todo set file usage to media ?
    // $this->fileUsage->add($file_entity, 'file', 'media', $mid);
    $waveform_options = \Drupal::config('audio_wavesurfer.settings')->get('waveform_options');
    $waveform_storage = $waveform_options['waveform_storage'];

    $mid = $entity->id();
    $field_media_audio_file = $entity->field_media_audio_file;
    if ($field_media_audio_file === NULL) {
      throw new \Exception('field_media_audio_file is NULL.');
    }

    foreach ($field_media_audio_file as $key => $item) {
      $fid = $item->target_id;
      $audio_clip_element = $item->audio_wavesurfer_clip;
      $clips = $audio_clip_element['clips'];

      foreach ($clips as $clip_name => $clip_attributes) {
        // Case where tabs infos is stored inside clips.
        if (!is_array($clip_attributes)) {
          continue;
        }

        $start_time = $clip_attributes['start_time'];
        $end_time = $clip_attributes['end_time'];

        $fid_in_audio_clip = $this->audioClipStorage->getQuery()
          ->accessCheck(TRUE)
          ->condition('original_id', $fid)
          ->condition('type', $clip_name)
          ->execute();

        if (empty($fid_in_audio_clip)) {
          $audio_clip = $this->audioClipService->createAudioClip($fid, $clip_name, $start_time, $end_time);
          $audio_clip_target_fid = $audio_clip->get('target_id')->value;
          if ($waveform_storage) {
            $this->waveformJsonService->buildWaveformJson($audio_clip_target_fid);
          }
          continue;
        }

        /** @var \Drupal\audio_clips\Entity\AudioClip $audio_clip_storage */
        $audio_clip_storage = $this->audioClipStorage->load(reset($fid_in_audio_clip));

        $start_time_value = $audio_clip_storage->get('start_time')->value;
        $end_time_value = $audio_clip_storage->get('end_time')->value;
        if ($start_time != $start_time_value || $end_time != $end_time_value) {
          $audio_clip = $this->audioClipService->udpateAudioClip($fid, $clip_name, $start_time, $end_time, $audio_clip_storage);
          $audio_clip_target_fid = $audio_clip->get('target_id')->value;
          if ($waveform_storage) {
            $this->waveformJsonService->buildWaveformJson($audio_clip_target_fid);
          }
        }
      }
    }
  }

}
