<?php

namespace Drupal\audio_wavesurfer\Form;

use Drupal\Core\Form\ConfigFormBase;
use Drupal\Core\Form\FormStateInterface;

/**
 * Configure wavesurfer parameters.
 */
class AudioWavesurferForm extends ConfigFormBase {

  /**
   * Config settings.
   *
   * @var string
   */
  const SETTINGS = 'audio_wavesurfer.settings';

  /**
   * {@inheritdoc}
   */
  public function getFormId() {
    return 'audio_wavesurfer_settings';
  }

  /**
   * {@inheritdoc}
   */
  protected function getEditableConfigNames() {
    return [
      static::SETTINGS,
    ];
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state) {
    $config = $this->config(static::SETTINGS);
    $color_options = $config->get('color_options');
    $bar_options = $config->get('bar_options');
    $waveform_options = $config->get('waveform_options');

    $form['vertical_tabs'] = [
      '#type' => 'vertical_tabs',
      '#title' => $this->t('Parameters'),
    ];

    $form['color_options'] = [
      '#type' => 'details',
      '#title' => $this->t('Color options'),
      '#group' => 'vertical_tabs',
    ];

    $form['color_options']['waveform_color'] = [
      '#type' => 'color',
      '#title' => $this->t('Waveform color'),
      '#default_value' => $color_options['waveform_color'],
    ];

    $form['color_options']['progress_color'] = [
      '#type' => 'color',
      '#title' => $this->t('Progress color'),
      '#default_value' => $color_options['progress_color'],
    ];

    $form['color_options']['cursor_color'] = [
      '#type' => 'color',
      '#title' => $this->t('Cursor color'),
      '#default_value' => $color_options['cursor_color'],
    ];

    $form['bar_options'] = [
      '#type' => 'details',
      '#title' => $this->t('Bar options'),
      '#group' => 'vertical_tabs',
    ];

    $form['bar_options']['bar_width'] = [
      '#type' => 'number',
      '#title' => $this->t('Bar width (px)'),
      '#default_value' => $bar_options['bar_width'],
    ];

    $form['bar_options']['bar_gap'] = [
      '#type' => 'number',
      '#title' => $this->t('Bar gap (px)'),
      '#default_value' => $bar_options['bar_gap'],
    ];

    $form['bar_options']['bar_radius'] = [
      '#type' => 'number',
      '#title' => $this->t('Bar radius (px)'),
      '#default_value' => $bar_options['bar_radius'],
    ];

    $form['waveform'] = [
      '#type' => 'details',
      '#title' => $this->t('Waveform'),
      '#group' => 'vertical_tabs',
    ];

    $form['waveform']['waveform_storage'] = [
      '#type' => 'checkbox',
      '#title' => $this->t('Waveform Storage'),
      '#description' => $this->t('Allow to store json of the audio waveform.
        This eliminates the need for the front end to calculate points, and therefore offers better performance.
        FFMpeg is necessary for this fonctionality. The JSON is created when audio file is added.'),
      '#default_value' => $waveform_options['waveform_storage'],
    ];

    $form['waveform']['waveform_points'] = [
      '#type' => 'number',
      '#title' => $this->t('Waveform Points'),
      '#description' => $this->t('Set the number of points in the waveform. The maximum is 2000 points, which offers a very good rendering..'),
      '#default_value' => $waveform_options['waveform_points'],
      '#max' => 2000,
    ];

    $form['waveform']['waveform_usage'] = [
      '#type' => 'checkbox',
      '#title' => $this->t('Waveform Usage'),
      '#description' => $this->t('Set if you want to use the stored waveform for rendering.'),
      '#default_value' => $waveform_options['waveform_usage'],
    ];

    $form['library'] = [
      '#type' => 'details',
      '#title' => $this->t('Library'),
      '#group' => 'vertical_tabs',
    ];

    $form['library']['library_path'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Library Path'),
      '#description' => $this->t('Library path if downloaded, otherwise leave blank. Installing it avoids loading via CDN.
      For library installation you can use npm: <a href="https://wavesurfer.xyz/docs/">WaveSurfer Library</a>'),
      '#default_value' => $config->get('library_path'),
    ];

    return parent::buildForm($form, $form_state);
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {
    $color_options = [
      'waveform_color' => $form_state->getValue('waveform_color'),
      'progress_color' => $form_state->getValue('progress_color'),
      'cursor_color' => $form_state->getValue('cursor_color'),
    ];

    $bar_options = [
      'bar_width' => $form_state->getValue('bar_width'),
      'bar_gap' => $form_state->getValue('bar_gap'),
      'bar_radius' => $form_state->getValue('bar_radius'),
    ];

    $waveform_options = [
      'waveform_storage' => $form_state->getValue('waveform_storage'),
      'waveform_points' => $form_state->getValue('waveform_points'),
      'waveform_usage' => $form_state->getValue('waveform_usage'),
    ];

    $this->config(static::SETTINGS)
      ->set('color_options', $color_options)
      ->set('bar_options', $bar_options)
      ->set('waveform_options', $waveform_options)
      ->set('library_path', $form_state->getValue('library_path'))
      ->save();

    parent::submitForm($form, $form_state);
  }

}
