<?php

namespace Drupal\audio_wavesurfer\Services;

use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\File\FileSystemInterface;
use Drupal\file\Entity\File;

/**
 * Service to manage waveform json.
 */
class WaveformJsonService {

  /**
   * The storage handler for the custom entity.
   *
   * @var \Drupal\Core\Entity\EntityStorageInterface
   */
  protected $storage;

  /**
   * Constructs the service.
   *
   * @param \Drupal\Core\Entity\EntityTypeManagerInterface $entity_type_manager
   *   The entity type manager service.
   */
  public function __construct(EntityTypeManagerInterface $entity_type_manager) {
    $this->storage = $entity_type_manager->getStorage('waveform');
  }

  /**
   * Build waveform JSON.
   *
   * @param int $fid
   *   The file ID.
   *
   * @return void
   *   Return nothing.
   *
   * @throws \Drupal\Core\Entity\EntityStorageException
   */
  public function buildWaveformJson(int $fid) {
    $file = File::load($fid);
    $file_path = $file->getFileUri();
    $waveform = $this->generateAudioWaveform($file->getFileUri());

    $path_parts = pathinfo($file_path);
    $filepath_target = $path_parts['dirname'] . '/' . $path_parts['filename'] . '.json';
    $filepath_real = \Drupal::service('file_system')->realpath($filepath_target);
    $json_content = json_encode($waveform);
    $file_system = \Drupal::service('file_system');

    $fid_in_waveform = $this->storage->getQuery()
      ->accessCheck(TRUE)
      ->condition('audio_fid', $fid)
      ->execute();

    $file_system->saveData($json_content, $filepath_real, FileSystemInterface::EXISTS_REPLACE);

    if (empty($fid_in_waveform)) {
      try {
        $file_entity = File::create([
          'uri' => $filepath_target,
          'status' => 1,
        ]);
        $file_entity->save();
      }
      catch (\Exception $e) {
        throw new \Exception('Error during writing JSON file : ' . $e->getMessage());
      }

      $this->storage->create([
        'audio_fid' => $fid,
        'waveform_fid' => $file_entity->id(),
      ])->save();
    }
  }

  /**
   * Generate Audio waveform.
   *
   * @param string $filepath
   *   The file path.
   *
   * @return array
   *   Return waveform points.
   */
  private function generateAudioWaveform(string $filepath) {
    $waveform_options = \Drupal::config('audio_wavesurfer.settings')->get('waveform_options');
    $waveform_points = 1000;

    if (array_key_exists('waveform_points', $waveform_options) && !empty($waveform_options['waveform_points'])) {
      $waveform_points = $waveform_options['waveform_points'];
    }

    $area_sample = 22050;
    $filepath_real = \Drupal::service('file_system')->realpath($filepath);
    $audio_duration = $this->getAudioDuration($filepath_real);
    $asetnsamples = intval(($audio_duration * $area_sample) / $waveform_points);

    $escaped_filepath = escapeshellarg($filepath_real);
    // areasample: frequency sampling
    // asetnsamples: sample block size
    // In this cas 10 points / sec.
    $command = "ffprobe -f lavfi -i amovie=$escaped_filepath,aresample=$area_sample,asetnsamples=$asetnsamples,astats=metadata=1:reset=1 -show_entries frame_tags=lavfi.astats.Overall.RMS_level -of json";
    $output = shell_exec($command);

    $data = json_decode($output, TRUE);
    $rmsValues = [];
    foreach ($data['frames'] as $frame) {
      if (isset($frame['tags']['lavfi.astats.Overall.RMS_level'])) {
        $rmsValues[] = (float) $frame['tags']['lavfi.astats.Overall.RMS_level'];
      }
    }

    // Transforms logarithmic values into linear ones.
    $linear_array = array_map(fn($db) => pow(10, $db / 20), $rmsValues);

    // Find max with 95e percentile.
    $sorted = $linear_array;
    sort($sorted);
    $percentile_index = (int) (count($sorted) * 0.95);
    $max = $sorted[$percentile_index];
    $min = min($linear_array);

    // Normalize with this max.
    $normalized_value = array_map(function ($x) use ($min, $max) {
      return max(0, min(1, ($x - $min) / ($max - $min)));
    }, $linear_array);

    return $this->removeNoiseOnArrayBorder($normalized_value);
  }

  /**
   * Get audio duration time in sec.
   *
   * @return float
   *   Return audio duration value.
   */
  private function getAudioDuration($file_path): float {
    if (!file_exists($file_path)) {
      throw new \InvalidArgumentException("File not found : $file_path");
    }

    $command = "ffprobe -i " . escapeshellarg($file_path) . " -show_entries format=duration -v quiet -of csv='p=0'";
    $duration = shell_exec($command);
    return (float) $duration;
  }

  /**
   * Get audio waveform points stored in json file.
   *
   * @param int $fid
   *   The audio file id.
   *
   * @return array|null
   *   Return waveform points or null.
   *
   * @throws \Drupal\Component\Plugin\Exception\InvalidPluginDefinitionException
   * @throws \Drupal\Component\Plugin\Exception\PluginNotFoundException
   */
  public function getAudioWaveformStored(int $fid): array|null {
    $query = \Drupal::entityQuery('waveform')
      ->accessCheck(TRUE)
      ->condition('audio_fid', $fid)
      ->execute();
    $waveform_storage = \Drupal::entityTypeManager()->getStorage('waveform');
    $waveform = $waveform_storage->load(reset($query));
    $waveform_fid = $waveform->waveform_fid->value;
    $waveform_file = File::load($waveform_fid);
    $waveform_file_path = $waveform_file->getFileUri();
    $waveform_peaks = NULL;
    if (file_exists($waveform_file_path)) {
      $json_data = file_get_contents($waveform_file_path);
      $waveform_peaks = json_decode($json_data, TRUE);
    }
    return $waveform_peaks;
  }

  /**
   * Replace 1 value by 0 at border of array.
   *
   * @param array $arr
   *   The array with noise.
   *
   * @return array
   *   The array without noise.
   */
  private function removeNoiseOnArrayBorder(array $arr): array {
    $n = count($arr);

    $i = 0;
    while ($i < $n && $arr[$i] === 1) {
      $arr[$i] = 0;
      $i++;
    }

    $j = $n - 1;
    while ($j >= 0 && $arr[$j] === 1) {
      $arr[$j] = 0;
      $j--;
    }

    return $arr;
  }

}
